# local dynamodb endpoint url
TEST_URL = "http://localhost:8000"

# dynamodb service string literal
DYNAMODB = 'dynamodb'

# source table literal
SOURCE_TABLE = 'source'

# producer config literal
PRODUCER = 'producer'

# consumer config literal
CONSUMER = 'consumer'

# dynamodb last evaluated key literal
LAST_EVALUATED_KEY = 'LastEvaluatedKey'

# thread count literal
THREAD_COUNT = 'TotalSegments'

# scan batch literal
SCAN_BATCH_SIZE = 'Limit'

# function literal
FUNCTION = 'function'

# timeout literal
TIMEOUT = 'timeout'

# args literal
ARGS = 'args'

# index name literal
INDEXNAME = 'IndexName'
